unit World;

interface

uses
  Tile, Graphics;

type
  TWorld = class
  private
    FTiles: array of TTile;
    FLevel: array of Integer;
    FGravity: Extended;
    FWidth: Integer;
    FHeight: Integer;
    FTileWidth: Integer;
    FTileHeight: Integer;
    procedure FreeAllTiles;
    function GetLevel(X, Y: Integer): TTile;
    function GetPixelHeight: Integer;
    function GetPixelWidth: Integer;
    function GetPixelLevel(X, Y: Integer): TTile;
  public
    constructor Create(aFileName: String);
    destructor Destroy; override;
    procedure Draw(Canvas: TCanvas; X, Y: Integer; sX, sY, eX, eY: Integer);
    property Level[X, Y: Integer]: TTile read GetLevel; default;
    property PixelLevel[X, Y: Integer]: TTile read GetPixelLevel;
    property Width: Integer read FWidth;
    property Height: Integer read FHeight;
    property TileWidth: Integer read FTileWidth;
    property TileHeight: Integer read FTileHeight;
    property Gravity: Extended read FGravity;
    property PixelWidth: Integer read GetPixelWidth;
    property PixelHeight: Integer read GetPixelHeight;
  end;

implementation

{ TWorld }

constructor TWorld.Create(aFileName: String);
var
  F: System.Text;
  I: Integer;
  TileNum: Integer;
  TilePicture: String;
  TileSolid: Byte;
  TileFriction: Extended;
begin
  AssignFile(F, aFileName);
  Reset(F);

  ReadLn(F, TileNum);
  ReadLn(F, FTileWidth);
  ReadLn(F, FTileHeight);
  SetLength(FTiles, TileNum);
  for I := 0 to TileNum - 1 do
  begin
    ReadLn(F, TilePicture);
    ReadLn(F, TileSolid);
    ReadLn(F, TileFriction);
    FTiles[I] := TTile.Create(TilePicture, Boolean(TileSolid), TileFriction);
  end;

  ReadLn(F, FGravity);
  ReadLn(F, FWidth);
  ReadLn(F, FHeight);
  SetLength(FLevel, FWidth * FHeight);
  for I := 0 to FWidth * FHeight - 1 do
  begin
    ReadLn(F, FLevel[I]);
  end;

  CloseFile(F);
end;

destructor TWorld.Destroy;
begin
  FreeAllTiles;
  FTiles := Nil;
  FLevel := Nil;
  inherited;
end;

procedure TWorld.Draw(Canvas: TCanvas; X, Y, sX, sY, eX, eY: Integer);
var
  cX, cY: Integer;
  cT: TTile;
begin
  for cY := sY to eY do
    for cX := sX to eX do
    begin
      cT := Level[cX ,cY];
      if cT.Picture <> nil then
      begin
        Canvas.Draw(X + (cX - sX) * FTileWidth,
                    Y + (cY - sY) * FTileHeight,
                    cT.Picture.Graphic);
      end;
    end;
end;

procedure TWorld.FreeAllTiles;
var
  I: Integer;
begin
  for I := 0 to Length(FTiles) - 1 do
    FTiles[I].Free;
end;

function TWorld.GetLevel(X, Y: Integer): TTile;
begin
  Result := FTiles[FLevel[X + Y * FWidth]];
end;

function TWorld.GetPixelHeight: Integer;
begin
  Result := FHeight * FTileHeight;
end;

function TWorld.GetPixelLevel(X, Y: Integer): TTile;
begin
  Result := Level[X div FTileWidth, Y div FTileHeight];
end;

function TWorld.GetPixelWidth: Integer;
begin
  Result := FWidth * FTileWidth;
end;

end.
